CREATE OR REPLACE PACKAGE BODY "tspkg_Update77To78" 
AS
	FUNCTION "fn_IsObjectExist"(p_ObjectName IN VARCHAR2)
	RETURN NUMBER
	IS
		IsExist NUMBER;
		ObjCount NUMBER;
	BEGIN
		BEGIN
		  EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM "USER_OBJECTS" WHERE "OBJECT_NAME" = ''' || p_ObjectName || '''' INTO ObjCount;
		  EXCEPTION
			WHEN OTHERS THEN
		  RETURN 0;
		END;
		IF ObjCount > 0 THEN
		  RETURN 1;
		END IF;
		RETURN 0;
	END;

	FUNCTION "fn_IsRowsExist"(p_TableName IN VARCHAR2)
	RETURN NUMBER
	IS
		IsTableExist NUMBER := "tspkg_Update77To78"."fn_IsObjectExist"(p_TableName);
		IsRowsExist NUMBER;
	BEGIN
		IF IsTableExist = 0 THEN
		  RETURN 0;
		END IF;
		EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM "' || p_TableName || '"' INTO IsRowsExist;
		IF IsRowsExist > 0 THEN
		  RETURN 1;
		END IF;
		RETURN 0;
	END;


	FUNCTION "fn_HasColumnUId"(p_TableName IN VARCHAR2)
	RETURN NUMBER
	IS
		v_Col_length NUMBER(8) := 0;
	BEGIN
		BEGIN
			SELECT utc.DATA_LENGTH INTO v_Col_length FROM USER_TAB_COLUMNS utc WHERE utc.TABLE_NAME = p_TableName AND utc.COLUMN_NAME = 'ColumnUId';
			EXCEPTION WHEN NO_DATA_FOUND THEN v_Col_length := 0;
		END;
		IF v_Col_length > 0
		THEN
			RETURN 1;
		END IF;
		RETURN 0;
	END;
	
    FUNCTION "fn_GetColumnUIdFromComment"(p_CommentValue IN VARCHAR2)
    RETURN VARCHAR2
    IS
		v_subStr VARCHAR2(1000) := '';
		v_startIndex NUMBER(8) := 0;
		v_endIndex NUMBER(8) := 0;
		v_result VARCHAR2(38) := '';
	BEGIN
		v_startIndex := INSTR(p_CommentValue, 'TS.EntitySchemaColumn.UId=');
		v_endIndex := INSTR(SUBSTR(p_CommentValue, v_startIndex), ';');
		v_subStr := SUBSTR(p_CommentValue, v_startIndex, v_endIndex);
		v_result := SUBSTR(v_subStr, INSTR(v_subStr, '=') + 1, 38);
		RETURN v_result;
	END;
	
	FUNCTION "tsp_GetReferenceSchemaUId"(p_CommentValue IN VARCHAR2)
    RETURN VARCHAR2
    IS
		v_subStr VARCHAR2(1000) := '';
		v_startIndex NUMBER(8) := 0;
		v_endIndex NUMBER(8) := 0;
		v_result VARCHAR2(38) := '';
	BEGIN
		v_startIndex := INSTR(p_CommentValue, 'TS.EntitySchemaColumn.ReferenceSchemaUId=');
		v_endIndex := INSTR(SUBSTR(p_CommentValue, v_startIndex), ';');
		v_subStr := SUBSTR(p_CommentValue, v_startIndex, v_endIndex);
		v_result := SUBSTR(v_subStr, INSTR(v_subStr, '=') + 1, 38);
		RETURN v_result;
	END;

	PROCEDURE "tsp_DropLczTables" (p_LczTableName IN VARCHAR2)
	IS
		v_count NUMBER(8) := 0;
		v_LczTableName VARCHAR2(250);
	BEGIN
		BEGIN
			SELECT COUNT(*) INTO v_count FROM USER_OBJECTS uo WHERE uo.OBJECT_NAME = p_LczTableName;
			EXCEPTION WHEN NO_DATA_FOUND THEN v_count := 0;
		END;
		IF (v_count > 0)
		THEN
			EXECUTE IMMEDIATE 'DROP TABLE "' || p_LczTableName || '"';
			v_count := 0;
		END IF;
		
		IF INSTR(p_LczTableName, 'LczOld') > 0
		THEN
			v_LczTableName := REPLACE(p_LczTableName, 'LczOld', 'Lcz');
			BEGIN
				SELECT COUNT(*) INTO v_count FROM USER_OBJECTS uo WHERE uo.OBJECT_NAME = v_LczTableName;
				EXCEPTION WHEN NO_DATA_FOUND THEN v_count := 0;
			END;
			IF (v_count > 0)
			THEN
				EXECUTE IMMEDIATE 'DROP TABLE "' || v_LczTableName || '"';
			END IF;
		END IF;
	END;

	PROCEDURE "tsp_RestoreTableData" (p_SourceName IN VARCHAR2, p_DestinationName IN VARCHAR2, p_PrimaryCulture IN VARCHAR2)
	IS
		v_command VARCHAR2(4000);
		v_updateScriptTemplate VARCHAR2(4000);
		v_columns VARCHAR2(4000);
		v_columnsCount NUMBER(8) := 0;
		v_ColumnName VARCHAR2(250);
		v_Col_length NUMBER(8) := 0;
		v_LczTableNameSuffix VARCHAR2(250);
		v_MainTableName VARCHAR2(250);
		v_TableName VARCHAR2(250);
		v_count NUMBER(8) := 0;
	BEGIN
		/* Check source table. */
		BEGIN
			SELECT COUNT(*) INTO v_count FROM USER_OBJECTS uo WHERE uo.OBJECT_NAME = p_SourceName;
			EXCEPTION WHEN NO_DATA_FOUND THEN v_count := 0;
		END;
		IF v_count = 0 THEN RETURN; END IF;
		BEGIN
			SELECT COUNT(*) INTO v_columnsCount FROM USER_TAB_COLUMNS utc WHERE utc.TABLE_NAME = p_SourceName;
			EXCEPTION WHEN NO_DATA_FOUND THEN v_columnsCount := 0;
		END;
		IF v_columnsCount = 0 THEN RETURN; END IF;
		/* Check destination table. */
		BEGIN
			SELECT COUNT(*) INTO v_count FROM USER_OBJECTS uo WHERE uo.OBJECT_NAME = p_DestinationName;
			EXCEPTION WHEN NO_DATA_FOUND THEN v_count := 0;
		END;
		
		IF v_count = 0
		THEN
			EXECUTE IMMEDIATE 'CREATE TABLE "' || p_DestinationName || '" AS SELECT *FROM "' || p_SourceName ||'"';
			RETURN;
		END IF;
		
		/* Restore missing destination table data. */
		DECLARE CURSOR ColumnsCursor IS 
				SELECT COLUMN_NAME FROM USER_TAB_COLUMNS utc WHERE utc.TABLE_NAME = p_SourceName;
			BEGIN
				OPEN ColumnsCursor;
				LOOP
					FETCH ColumnsCursor INTO v_ColumnName;
					EXIT WHEN ColumnsCursor%NOTFOUND;
					IF v_columns IS NULL
					THEN
						v_columns := '"' || v_ColumnName || '"';
					ELSE 
						v_columns := v_columns || ', "' || v_ColumnName || '"';
					END IF;
				END LOOP;
			END;
	
		EXECUTE IMMEDIATE 'SELECT ss."Name" "TableName" FROM USER_COL_COMMENTS ucc, "SysSchema" ss
			WHERE ucc.TABLE_NAME = ''' || p_DestinationName || ''' AND ucc.COLUMN_NAME = ''RecordId''
				AND ss."UId" = "tspkg_Update77To78"."tsp_GetReferenceSchemaUId"(ucc.COMMENTS)' INTO v_TableName;
		
		v_command := 'INSERT INTO "' || p_DestinationName || '" ('|| v_columns ||') 
			SELECT '|| v_columns ||' FROM "' || p_SourceName || '" sn WHERE sn."Id" NOT IN (SELECT "Id" FROM "' || p_DestinationName || '")
				AND EXISTS (SELECT 1 FROM "' || v_TableName || '" rt WHERE rt."Id" = sn."RecordId")
				AND NOT EXISTS (SELECT 1 FROM "' || p_DestinationName || '" rtt 
			WHERE rtt."SysCultureId" = sn."SysCultureId" AND rtt."RecordId" = sn."RecordId")
				AND (sn."RecordId", sn."SysCultureId") IN (SELECT "RecordId", "SysCultureId" FROM "' || p_SourceName || 
				'" GROUP BY "RecordId", "SysCultureId" HAVING COUNT(*) = 1)';
		EXECUTE IMMEDIATE v_command;
		
		/* Update existing destination table data. */
		v_updateScriptTemplate := 'UPDATE "##Destination##" d SET d."##Column##" = (SELECT s."##Column##" FROM "##Source##" s WHERE s."Id" = d."Id") WHERE EXISTS (SELECT 1 FROM "##Source##" s WHERE s."Id" = d."Id")';
	 
		DECLARE CURSOR ColumnsCursor IS
				SELECT COLUMN_NAME FROM USER_TAB_COLUMNS utc WHERE utc.TABLE_NAME = p_SourceName
				AND COLUMN_NAME NOT IN ('Id', 'CreatedOn', 'CreatedById', 'ModifiedOn', 'ModifiedById', 'ProcessListeners', 'RecordId', 'ColumnUId', 'SysCultureId');
			BEGIN
				OPEN ColumnsCursor;
				LOOP
					FETCH ColumnsCursor INTO v_ColumnName;
					EXIT WHEN ColumnsCursor%NOTFOUND;
					v_command := REPLACE(v_updateScriptTemplate, '##Destination##', p_DestinationName);
					v_command := REPLACE(v_command, '##Source##', p_SourceName);
					v_command := REPLACE(v_command, '##Column##', v_ColumnName);	
					EXECUTE IMMEDIATE v_command;
				END LOOP;
			END;
			
		
		BEGIN /* Update main table data. */

			IF "fn_HasColumnUId"(p_DestinationName) = 1
			THEN
				RETURN;
			END IF;

			v_updateScriptTemplate := 'UPDATE "##MainTable##" m SET m."##Column##" = 
				(SELECT d."##Column##" FROM "##Destination##" d WHERE d."RecordId" = m."Id" AND ROWNUM = 1) 
				WHERE EXISTS (SELECT 1 FROM "##Destination##" ds WHERE ds."RecordId" = m."Id" 
					AND ds."SysCultureId" = ''' || p_PrimaryCulture || ''')';
			DECLARE CURSOR ColumnsCursor IS 
					SELECT COLUMN_NAME FROM USER_TAB_COLUMNS utc WHERE utc.TABLE_NAME = p_SourceName
					AND COLUMN_NAME NOT IN ('Id', 'CreatedOn', 'CreatedById', 'ModifiedOn', 'ModifiedById', 'ProcessListeners', 'RecordId', 'ColumnUId', 'SysCultureId');
				BEGIN
					OPEN ColumnsCursor;
					LOOP
						FETCH ColumnsCursor INTO v_ColumnName;
						EXIT WHEN ColumnsCursor%NOTFOUND;
						v_command := REPLACE(v_updateScriptTemplate, '##Destination##', p_DestinationName);
						v_command := REPLACE(v_command, '##MainTable##', v_TableName);
						v_command := REPLACE(v_command, '##Column##', v_ColumnName);	
						EXECUTE IMMEDIATE v_command;
					END LOOP;
				END;
		END;

		BEGIN /* Delete primary culture localization values. */

			IF "fn_HasColumnUId"(p_DestinationName) = 1
			THEN
				RETURN;
			END IF;

			v_command := 'DELETE FROM "' || p_DestinationName || '" WHERE "SysCultureId" = ''' || p_PrimaryCulture || '''';
			EXECUTE IMMEDIATE v_command;

		END;		
		
	END;
	
END "tspkg_Update77To78";
